from apiaudio.helper_classes import UpdatableResource, ListableResource, DeletableResource


class LexiItem(UpdatableResource, ListableResource, DeletableResource):
    OBJECT_NAME = "diction/custom/item"
    resource_path = "/diction/custom/item"

class CustomDict(ListableResource):
    OBJECT_NAME = "diction/custom"
    resource_path = "/diction/custom"

class Lexi(ListableResource):
    OBJECT_NAME = "diction"
    resource_path = "/diction"

    custom_word_path = resource_path + "/custom/item"
    list_words = resource_path + "/custom"
    
    @classmethod
    def register_custom_word(cls, word, replacement, lang, specialization="default", contentType="basic"):
        return LexiItem.update(
            **{
                "word" : word, 
                "replacement" : replacement, 
                "lang" : lang,
                "specialization" : specialization,
                "contentType" : contentType
            }
        )

    @classmethod
    def list_custom_words(cls, **args):
        return LexiItem.list(**args)
    
    @classmethod
    def list_custom_dicts(cls, **args):
        return CustomDict.list(**args)
    
    @classmethod
    def delete_custom_word(cls, **args):
        return LexiItem.delete(**args)




from apiaudio.helper_classes import (
    CreatableResource,
    DeletableResource,
    ListableResource,
    RetrievableResource,
)


class Script(
    ListableResource, CreatableResource, RetrievableResource, DeletableResource
):
    OBJECT_NAME = "script"
    resource_path = "/script"
    random_url = "/script/random"

    @classmethod
    def get_random_text(cls, category=None):
        return cls._get_request(
            path_param=cls.random_url, request_params={"category": category}
        )

    @classmethod
    def preview(cls, scriptId, voice):
        params = {"preview": True, "voice": voice}
        r = cls._get_request(
            path_param=cls.resource_path + f"/{scriptId}", request_params=params
        )

        if "scriptText" in r:
            res = {"preview": r["scriptText"]}
            if "wordsNotInDict" in r:
                res["wordsNotInDict"] = r["wordsNotInDict"]

            return res
        else:  # in practice this won't happen as _get_request raises an exception
            return "PREVIEW FAILED"



import apiaudio
import requests
import shutil
import os
import requests
import re

from requests.exceptions import HTTPError
from . import sdk_version


class APIRequest:
    def _api_key_checker(api_key=None):
        api_key_error_message = "Please specify a valid api_key or create one here:\n https://console.api.audio"
        PLACEHOLDERS = ["your-key", "APIKEY", "API_KEY"]
        if api_key == None or api_key in PLACEHOLDERS:
            # aflr_key is for backward compatibility with the old name of env vars.
            api_key = os.environ.get("apiaudio_key", os.environ.get("aflr_key"))

            if api_key == None:
                raise TypeError(f"No api_key has been found. {api_key_error_message}")

        if not isinstance(api_key, str):
            raise TypeError("api_key must be of type string.")

        if len(api_key) < 32:
            raise ValueError(
                f"api_key has less than 32 characters. {api_key_error_message}"
            )
        apiaudio.api_key = api_key
        return

    def set_assume_org_id(org_id):
        apiaudio.assume_org_id = org_id

    @classmethod
    def _build_header(cls):
        cls._api_key_checker(apiaudio.api_key)
        return {
            "x-api-key": apiaudio.api_key,
            "x-python-sdk-version": sdk_version,
            "x-assume-org": apiaudio.assume_org_id,
        }

    @classmethod
    def _post_request(cls, json, url=None, headers=None):
        headers = headers or {}
        loop_status_code = cls.__dict__.get("loop_status_code")
        url = url or f"{apiaudio.api_base}{cls.resource_path}"
        headers.update(cls._build_header())
        r = requests.post(url=url, headers=headers, json=json)

        if loop_status_code:
            while r.status_code == loop_status_code:
                apiaudio._logger.info(
                    f"{cls.OBJECT_NAME.upper()}: Creation in progress..."
                )
                r = requests.get(url=url, headers=headers, params=json)

        cls._expanded_raise_for_status(r)

        if r.headers["Content-Type"] != "application/json":
            return r.content
        return r.json()

    @classmethod
    def _delete_request(cls, url=None, path_param=None, request_params=None):
        url = url or f"{apiaudio.api_base}{cls.resource_path}"

        headers = cls._build_header()
        if path_param:
            url = f"{apiaudio.api_base}{path_param}"

        if request_params:
            r = requests.delete(url=url, headers=headers, params=request_params)
        else:
            r = requests.delete(url=url, headers=headers)

        cls._expanded_raise_for_status(r)

        return r.json()

    @classmethod
    def _put_request_s3(cls, data, url=None, headers=None):
        url = url or f"{apiaudio.api_base}{cls.resource_path}"

        r = requests.put(url=url, headers=headers, data=data)

        cls._expanded_raise_for_status(r)

        if r.status_code != 200:
            raise ValueError("Error performing the PUT request")

        # since aws s3 does not return a body on PUT requests,
        # r.json() does not work here
        return r
    
    @classmethod
    def _put_request(cls, json, url=None, headers=None):
        headers = headers or {}
        url = url or f"{apiaudio.api_base}{cls.resource_path}"
        headers.update(cls._build_header())
        
        r = requests.put(url=url, headers=headers, json=json)

        cls._expanded_raise_for_status(r)

        if r.headers["Content-Type"] != "application/json":
            return r.content
        return r.json()

    @classmethod
    def _get_request(cls, url=None, path_param=None, request_params=None):
        loop_status_code = cls.__dict__.get("loop_status_code")
        url = url or f"{apiaudio.api_base}{cls.resource_path}"

        headers = cls._build_header()

        if path_param:
            url = f"{apiaudio.api_base}{path_param}"

        if request_params:
            r = requests.get(url=url, headers=headers, params=request_params)
        else:
            r = requests.get(url=url, headers=headers)

        if loop_status_code:
            while r.status_code == loop_status_code:
                apiaudio._logger.info(
                    f"{cls.OBJECT_NAME.upper()}: Waiting for resource to be retrieved..."
                )
                if request_params:
                    r = requests.get(url=url, headers=headers, params=request_params)
                else:
                    r = requests.get(url=url, headers=headers)

        cls._expanded_raise_for_status(r)

        return r.json()

    @classmethod
    def _download_request(cls, url, destination, version=""):
        loop_status_code = cls.__dict__.get("loop_status_code")
        if type(url) is not str:
            raise TypeError("Error retrieving the audio files.")

        remote_filename = url.split("/")[-1].split("?")[0]
        local_filename = f"{destination}/{remote_filename}"
        local_filename = local_filename.replace("%243ct10n", "section")
        local_filename = local_filename.replace("%7C", "|")

        r = requests.get(url, stream=True)
        if loop_status_code:
            while r.status_code == loop_status_code:
                apiaudio._logger.info(
                    f"{cls.OBJECT_NAME.upper()}: Waiting for resource to be downloaded..."
                )
                r = requests.get(url, stream=True)

        cls._expanded_raise_for_status(r)

        with open(local_filename, "wb") as f:
            shutil.copyfileobj(r.raw, f)

        return local_filename

    @classmethod
    def config_test(cls):
        return f"Configured to transact {cls.OBJECT_NAME} objects to {apiaudio.api_base}{cls.resource_path} with api_key = {apiaudio.api_key}"

    @classmethod
    def _expanded_raise_for_status(self, res):
        """
        Take a "requests" response object and expand the raise_for_status method to return errors from API
        @param res:
        @return: None
        """
        try:
            latest_version = res.headers.get("x-sdk-latest", apiaudio.sdk_version)
            if (
                latest_version != apiaudio.sdk_version
                and not apiaudio._version_warning_issued
            ):
                apiaudio._logger.warning(
                    f"The latest version of apiaudio is {latest_version} and the version you're using is {apiaudio.sdk_version}. Consider upgrading, as you might be missing out on new features and bug fixes."
                )
                apiaudio._version_warning_issued = True

            if res.headers.get("Warning"):
                for warn in re.findall(
                    r"\"(.*?)\"", res.headers["Warning"]
                ):  # get all messages in between ""
                    apiaudio._logger.warning(f"{self.OBJECT_NAME.upper()}: {warn}")

            res.raise_for_status()
        except HTTPError as e:
            if res.json():
                raise HTTPError(
                    "{} \n Error Message from API: \n {}".format(res.json(), str(e))
                )
            else:
                raise e
        return

    @classmethod
    def _options_request(cls, url=None):
        url = url or f"{apiaudio.api_base}{cls.resource_path}"
        headers = cls._build_header()

        r = requests.options(url=url, headers=headers)

        cls._expanded_raise_for_status(r)

        return r.json()



import apiaudio
import pprint
import pytest
import os
import logging
from time import time

apiaudio.api_key = os.environ["AFLR_API_KEY"]
#apiaudio.api_base = "https://staging-v1.api.audio"


def test_level_setting():
    assert apiaudio._logger.level is logging.WARNING
    apiaudio.set_logger_level("DEBUG")
    assert apiaudio._logger.level is logging.DEBUG


def test_list_parameters():
    assert len(apiaudio.Voice.list_parameters()) > 0


def test_sound_design():
    sounds = apiaudio.Sound.list()
    print(sounds)


def test_create_scripts():
    create_script = apiaudio.Script.create(
        scriptId="test_sdk",
        scriptText="<<sectionName::question>> Hey! Do you know we support multiple voices from different providers in the same script? I am a polly voice from Amazon. <<sectionName::answer>> I am Azure voice from Microsoft. I think Azure voices sound awesome.",
    )
    get_script = apiaudio.Script.retrieve(scriptId="test_sdk")
    apiaudio.Speech.create(scriptId="test_sdk", voice="brandon")


def test_create_mastering():
    create_script = apiaudio.Script.create(
        scriptText="<<sectionName::question>> Hey! Do you know we support multiple voices from different providers in the same script? I am a polly voice from Amazon. <<sectionName::answer>> I am Azure voice from Microsoft. I think Azure voices sound awesome."
    )
    scriptId = create_script.get("scriptId")
    apiaudio.Speech.create(scriptId=scriptId, voice="brandon")
    mastering = apiaudio.Mastering.create(scriptId=scriptId)
    assert len(mastering) > 0
    assert len(mastering["url"]) > 0


def test_birdcache():
    birdcache = apiaudio.Birdcache.create(
        type="mastering",
        voice="linda",
        text="This is {{username|me}} creating synchronous text to speech",
        audience={"username": ["salih", "sam", "timo"]},
        soundTemplate="openup",
    )
    assert len(birdcache) > 0
    assert birdcache[1]["text"] == "This is salih creating synchronous text to speech"


def test_script_versions():
    from .assets import test_versions

    script = apiaudio.Script.create(
        scriptId="test_sdk", scriptText="hello...", versions=test_versions
    )

    assert script.get("availableVersions") == ["v0"] + [
        str(x) for x in test_versions.keys()
    ]

    for ver in test_versions:
        speech = apiaudio.Speech.create(scriptId=script.get("scriptId"), version=ver)
        key = str(ver) + "|default"
        assert key in speech.keys()
        assert speech[key]["status_code"] == "201"


def test_synctts():
    audio = apiaudio.SyncTTS.create(
        text="Hello, test 123!",
        voice="joanna"
    )
    assert isinstance(audio, bytes)
    assert b"RIFF" in audio

    d = apiaudio.SyncTTS.create(
        text="Hello, test 123.",
        voice="joanna",
        url=True
    )
    assert isinstance(d, dict)
    assert "api.audio" in d.get("url", [])

    mp3_audio = apiaudio.SyncTTS.create(
        text="Hello, test 123.",
        voice="joanna",
        format="mp3"
    )
    assert isinstance(mp3_audio, bytes)
    assert not b"RIFF" in mp3_audio

def test_processing_loop_speech():
    t0 = time()
    speech = apiaudio.Speech.create(scriptId="longProcessing", voice="Dieter")
    assert len(speech) == 3  # number of sections in the script
    assert time() - t0 > 30


def test_processing_loop_mastering():
    t0 = time()
    apiaudio.Script.create(
        scriptId="test_sdk",
        scriptText="<<sectionName::question>> Hey! Do you know we support multiple voices from different providers in the same script? I am a polly voice from Amazon. <<sectionName::answer>> I am Azure voice from Microsoft. I think Azure voices sound awesome.",
    )
    apiaudio.Speech.create(scriptId="test_sdk", voice="brandon")
    mastering = apiaudio.Mastering.create(scriptId="test_sdk", forceLength=700)
    print(mastering)
    assert "url" in mastering
    assert time() - t0 > 30



# Aflorithmic Python bindings
# API docs at https://docs.api.audio/

# Configuration variables

sdk_version = "0.16.6"

api_key = None
assume_org_id = None
api_base = "https://v1.api.audio"
upload_api_base = "https://file.api.audio"  # not implemented yet.

# future
api_version = None
verify_ssl_certs = True
proxy = None
default_http_client = None
app_info = None
enable_telemetry = True
max_network_retries = 0

# API resources
from apiaudio.api_resources import *
from apiaudio.api_request import APIRequest

# logging
from apiaudio.logging import SDKLogger


_logger = SDKLogger()
_version_warning_issued = False


def set_logger_level(level):
    available_levels = ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]
    level = level.upper()
    assert level in available_levels, f"Available logging levels: {available_levels}"

    _logger = SDKLogger(level=level)


# super org assume mechanism
def set_assume_org_id(org_id):
    return APIRequest.set_assume_org_id(org_id)



from apiaudio.helper_classes import CreatableResource


class SyncTTS(CreatableResource):
    OBJECT_NAME = "SyncTTS"
    resource_path = "/speech/sync"

    @classmethod
    def create(cls, **params):
        headers = {"Accept": "audio/wav"}
        if params.get("format"):
            headers = {"Accept": f"audio/{params.pop('format')}"}
        return cls._post_request(json=params, headers=headers)



from apiaudio.helper_classes import CreatableResource, HelpResource


class CreateMediaWithSound(CreatableResource):
    OBJECT_NAME = "orchestrator"
    resource_path = "/orchestrator/media_with_sound"


class CreateAudio(CreatableResource):
    OBJECT_NAME = "orchestrator"
    resource_path = "/orchestrator/create_audio"


class CreateThreeSections(CreatableResource):
    OBJECT_NAME = "orchestrator"
    resource_path = "/orchestrator/create_three_sections"


class Orchestrator(CreatableResource, HelpResource):
    OBJECT_NAME = "orchestrator"
    resource_path = "/orchestrator"

    @classmethod
    def create_media_with_sound(cls, **args):
        return cls.download(cls, CreateMediaWithSound.create(**args))

    @classmethod
    def create_audio(cls, **args):
        return cls.download(cls, CreateAudio.create(**args))

    @classmethod
    def create_three_sections(cls, **args):
        return cls.download(cls, CreateThreeSections.create(**args))

    def download(cls, response):
        local_filename = cls._download_request(url=response.get("url"), destination=".")
        return local_filename



from apiaudio.helper_classes import (
    CreatableResource,
    DownloadableResource,
    RetrievableResource,
)


class Mastering(CreatableResource, RetrievableResource, DownloadableResource):
    OBJECT_NAME = "mastering"
    resource_path = "/mastering"
    mastering_preset_list_path = resource_path + "-presets"

    loop_status_code = 202

    @classmethod
    def list_presets(cls):
        return cls._get_request(path_param=cls.mastering_preset_list_path)



from apiaudio.api_request import APIRequest


class Connector(APIRequest):
    OBJECT_NAME = "connector"
    resource_path = "/connector/"
    connection_path = "/connection/"

    @classmethod
    def retrieve(cls, name):
        if not name:
            raise Exception("Name must be set")
        return cls._get_request(path_param=cls.resource_path + name)

    @classmethod
    def connection(cls, connection_id):
        if not connection_id:
            raise Exception("Connection id must be set")
        return cls._get_request(path_param=cls.connection_path + connection_id)



import logging


class CustomFormatter(logging.Formatter):

    # For debugging: "%(asctime)s - %(name)s - %(levelname)s - %(message)s (%(filename)s:%(lineno)d)"
    def string_format(color):
        ANSI = {
            "grey": "\x1b[38;20m",
            "yellow": "\x1b[33;20m",
            "red": "\x1b[31;20m",
            "bold_red": "\x1b[31;1m",
            "bold": "\033[1m",
            "reset": "\x1b[0m",
        }
        return f"{ANSI[color]}[%(levelname)s] {ANSI['bold']}%(name)s{ANSI['reset'] + ANSI[color]}: %(message)s{ANSI['reset']}"

    FORMATS = {
        logging.DEBUG: string_format("grey"),
        logging.INFO: string_format("grey"),
        logging.WARNING: string_format("yellow"),
        logging.ERROR: string_format("red"),
        logging.CRITICAL: string_format("bold_red"),
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)


def SDKLogger(level="WARNING"):
    """
    Add a stream handler for the given name and level to the logging module.
    By default, this logs all apiaudio messages to ``stdout``.

    For debugging, set level=logging.DEBUG

    :type name: string
    :param name: Log name
    :type level: int
    :param level: Logging level, e.g. ``logging.INFO``
    """
    logger = logging.getLogger("apiaudio")
    logger.setLevel(getattr(logging, level))

    # create console handler with a higher log level
    handler = logging.StreamHandler()
    handler.setLevel(level)

    handler.setFormatter(CustomFormatter())

    logger.addHandler(handler)
    return logger
